@@ -312,16 +312,27 @@
 #define CI_APCDelayBeforeShutdown        65
 #define    APC_CMD_DelayBeforeShutdown    0
 
+/* Only seen on the BackUPS Pro USB (so far) */
+#define CI_BUPBattCapBeforeStartup       66
+#define    BUP_CMD_BattCapBeforeStartup   0
+#define CI_BUPDelayBeforeStartup         67
+#define    BUP_CMD_DelayBeforeStartup     0
+#define CI_BUPSelfTest                   68
+#define    BUP_CMD_SelfTest               0
+#define CI_BUPHibernate                  69
+#define    BUP_CMD_Hibernate              0
+
+
 /* Items below this line are not "probed" for */
-#define CI_CYCLE_EPROM         66     /* Cycle programmable EPROM values */
+#define CI_CYCLE_EPROM         70     /* Cycle programmable EPROM values */
 #define    APC_CMD_CYCLE_EPROM    '-'
-#define CI_UPS_CAPS            67     /* Get UPS capabilities (command) string */
+#define CI_UPS_CAPS            71     /* Get UPS capabilities (command) string */
 #define    APC_CMD_UPS_CAPS       'a'
 /* ^^^^^^^^^^ see below if you change this ^^^^^^ */
 /* set to last command index. CHANGE!!! when adding new code.  */
 /* vvvvvvvvvv change here vvvvvvvvvvvvvvvvvvvvvvv */
 #define CI_MAXCI         CI_UPS_CAPS      /* maximum UPS commands we handle */
-#define CI_MAX_CAPS      CI_APCDelayBeforeShutdown
+#define CI_MAX_CAPS      CI_BUPHibernate
 
 #define GO_ON_BATT              'W'
 #define GO_ON_LINE              'X'
