@@ -11,49 +11,23 @@
  */
 
 /*
- *                     GNU GENERAL PUBLIC LICENSE
- *                        Version 2, June 1991
- *
- *  Copyright (C) 1989, 1991 Free Software Foundation, Inc.
- *                           675 Mass Ave, Cambridge, MA 02139, USA
- *  Everyone is permitted to copy and distribute verbatim copies
- *  of this license document, but changing it is not allowed.
- *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
- *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
- *
- */
+   Copyright (C) 2000-2004 Kern Sibbald
+
+   This program is free software; you can redistribute it and/or
+   modify it under the terms of the GNU General Public License as
+   published by the Free Software Foundation; either version 2 of
+   the License, or (at your option) any later version.
+
+   This program is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
+   General Public License for more details.
+
+   You should have received a copy of the GNU General Public
+   License along with this program; if not, write to the Free
+   Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
+   MA 02111-1307, USA.
 
-/*
- *  IN NO EVENT SHALL ANY AND ALL PERSONS INVOLVED IN THE DEVELOPMENT OF THIS
- *  PACKAGE, NOW REFERRED TO AS "APCUPSD-Team" BE LIABLE TO ANY PARTY FOR
- *  DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING
- *  OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF ANY OR ALL
- *  OF THE "APCUPSD-Team" HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
- *
- *  THE "APCUPSD-Team" SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING,
- *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
- *  FITNESS FOR A PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS
- *  ON AN "AS IS" BASIS, AND THE "APCUPSD-Team" HAS NO OBLIGATION TO PROVIDE
- *  MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
- *
- *  THE "APCUPSD-Team" HAS ABSOLUTELY NO CONNECTION WITH THE COMPANY
- *  AMERICAN POWER CONVERSION, "APCC".  THE "APCUPSD-Team" DID NOT AND
- *  HAS NOT SIGNED ANY NON-DISCLOSURE AGREEMENTS WITH "APCC".  ANY AND ALL
- *  OF THE LOOK-A-LIKE ( UPSlink(tm) Language ) WAS DERIVED FROM THE
- *  SOURCES LISTED BELOW.
- *
  */
 
 /*********************************************************************/
@@ -168,8 +142,8 @@ typedef struct SELFTEST {
 
 
 typedef struct geninfo {
-    char *name;             /* JHNC: name mustn't contain whitespace */
-    char *long_name;
+    const char *name;             /* JHNC: name mustn't contain whitespace */
+    const char *long_name;
     int type;
 } GENINFO; /* for static declaration of data */
 
@@ -240,7 +214,7 @@ typedef struct netdata {
 #define UPSINFO_ID "UPS!"
 
 typedef struct upsinfo {
-    char id[4];
+    char id[5];
     int version;
     int size;
     char release[20];
@@ -404,7 +378,7 @@ typedef struct upsinfo {
     int refcnt;                       /* thread attach count */
 #endif
 
-    struct upsdriver *driver;       /* UPS driver for this UPSINFO */
+    const struct upsdriver *driver; /* UPS driver for this UPSINFO */
     void *driver_internal_data;     /* Driver private data */
 } UPSINFO;
 
@@ -418,13 +392,13 @@ typedef struct datainfo {
     char accessmagic[ACCESS_MAGIC_SIZE];
 } DATAINFO;
 
-typedef int (HANDLER) (UPSINFO *, size_t, GENINFO *, const char *);
+typedef int (HANDLER) (UPSINFO *, int, const GENINFO *, const char *);
 
 typedef struct {
     const char *key;
     HANDLER *handler;
     size_t offset;
-    GENINFO *values;
+    const GENINFO *values;
     const char *help;
 } PAIRS;
 
