@@ -9,93 +9,23 @@
  */
 
 /*
- *			 GNU GENERAL PUBLIC LICENSE
- *			    Version 2, June 1991
- *
- *  Copyright (C) 1989, 1991 Free Software Foundation, Inc.
- *			       675 Mass Ave, Cambridge, MA 02139, USA
- *  Everyone is permitted to copy and distribute verbatim copies
- *  of this license document, but changing it is not allowed.
- *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
- *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
- *
- */
+   Copyright (C) 1999-2004 Kern Sibbald
 
-/*
- *  IN NO EVENT SHALL ANY AND ALL PERSONS INVOLVED IN THE DEVELOPMENT OF THIS
- *  PACKAGE, NOW REFERRED TO AS "APCUPSD-Team" BE LIABLE TO ANY PARTY FOR
- *  DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING
- *  OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF ANY OR ALL
- *  OF THE "APCUPSD-Team" HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
- *
- *  THE "APCUPSD-Team" SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING,
- *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
- *  FITNESS FOR A PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS
- *  ON AN "AS IS" BASIS, AND THE "APCUPSD-Team" HAS NO OBLIGATION TO PROVIDE
- *  MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
- *
- *  THE "APCUPSD-Team" HAS ABSOLUTELY NO CONNECTION WITH THE COMPANY
- *  AMERICAN POWER CONVERSION, "APCC".  THE "APCUPSD-Team" DID NOT AND
- *  HAS NOT SIGNED ANY NON-DISCLOSURE AGREEMENTS WITH "APCC".  ANY AND ALL
- *  OF THE LOOK-A-LIKE ( UPSlink(tm) Language ) WAS DERIVED FROM THE
- *  SOURCES LISTED BELOW.
- *
- */
+   This program is free software; you can redistribute it and/or
+   modify it under the terms of the GNU General Public License as
+   published by the Free Software Foundation; either version 2 of
+   the License, or (at your option) any later version.
 
-/*
- *  Parts of the code are from Miquel van Smoorenburg's powerd.c
- *  Other parts are original from Christian Holtje <docwhat@uiuc.edu>
- *  Smart-UPS code from Pavel Korensky's apcd.c, apcd.h, and upsnet.c.
- *  I believe that it is okay to say that this is Public Domain, just
- *  give credit, where credit is due.
- *
- *  Disclaimer:  We make NO claims to this software, and take no
- *               resposibility for it's use/misuse.
- *
- *  Cable for a smarter Back-UPS and SmartUPS from APC only.
- *
- *  Computer Side   |  Description of Cable		 |  UPS Side
- *  DB9f  |  DB25f  |					   |	DB9m
- *   4		|   20	  |  DTR (5vcc) 	     *below  |	  n/c
- *   8		|    5	  |  CTS (low-batt)		 *below  |     5
- *   2		|    3	  |  RxD (other line-fail)  *below  |	  3
- *   5		|    7	  |  Ground (Signal)			  |	4
- *   1		|    8	  |  CD (line-fail from ups)		  |	2
- *   7		|    4	  |  RTS (shutdown ups) 	     |	   1
- *  n/c   |    1    |  Frame/Case Gnd (optional)      |     9
- *
- *  The "*below" one needs solder type DB connectors with standard
- *  hoods, two (2) 4.7K ohm 1/4 watt %5, one (1) foot of 3/32" (inch)
- *  shrink wrap, rosin core solder, and three (3) to five (5) feet of
- *  22AWG multi-stranded five (5) conductor cable. 
- *
- *  9/20/96 2:00am
- *
- */
+   This program is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
+   General Public License for more details.
+
+   You should have received a copy of the GNU General Public
+   License along with this program; if not, write to the Free
+   Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
+   MA 02111-1307, USA.
 
-/*
- * Add here an identifer that is unique for tracking changes
- * made by each person other than myself.
- *
- * Christopher J. Reimer <reimer@doe.carleton.ca>	 CJR
- * Werner Panocha <wpanocha@t-online.de>		w.p.
- * Brian Schau <bsc@fleggaard.dk>			 BSC
- * Marko Sakari Asplund <Marko.Asplund@cern.ch> 	MSA
- * Riccardo Facchetti <riccardo@master.oasi.gpa.it>			   -RF
- * Jonathan H N Chin <jc254@newton.cam.ac.uk>		     JHNC
- * Kern Sibbald <kern@sibbald.com>			  KES
  */
 
 /* The big overall flow of apcupsd is as follows
@@ -148,6 +78,7 @@ char argvalue[MAXSTRING];
 static void daemon_start(void);
 
 int shm_OK = 0;
+extern int kill_on_powerfail;
 
 /**********************************************************************
  * the terminate function and trapping signals allows apcupsd
@@ -198,7 +129,7 @@ void apcupsd_error_cleanup(UPSINFO *ups)
  *  and exits. It is normally called from the Error_abort
  *  define, which inserts the file and line number.
  */
-void apcupsd_error_out(char *file, int line, char *fmt,...)
+void apcupsd_error_out(const char *file, int line, const char *fmt,...)
 {
     char      buf[256];
     va_list   arg_ptr;
@@ -218,7 +149,7 @@ void apcupsd_error_out(char *file, int line, char *fmt,...)
 /* subroutine error_exit simply prints the supplied error
  * message, cleans up, and exits
  */
-void apcupsd_error_exit(char *fmt,...)
+void apcupsd_error_exit(const char *fmt,...)
 {
     char      buf[256];
     va_list   arg_ptr;
@@ -277,8 +208,7 @@ int main(int argc, char *argv[]) {
      * which requires initialization.
      */
     init_proctitle(argv[0]);
-    strncpy(argvalue, argv[0], sizeof(argvalue)-1);
-    argvalue[sizeof(argvalue)-1] = 0;
+    astrncpy(argvalue, argv[0], sizeof(argvalue));
       
     ups = new_ups();		      /* get new ups */
     if (!ups) {
@@ -318,6 +248,18 @@ int main(int argc, char *argv[]) {
     check_for_config(ups, cfgfile);
     Dmsg1(10, "Config file %s processed.\n", cfgfile);
 
+    /*
+     * Disallow --kill-on-powerfail in conjunction with simple signaling
+     * UPSes. Such UPSes have no shutdown grace period so using --kill-on-
+     * powerfail would guarantee an unclean shutdown.
+     */
+    if (kill_on_powerfail && ups->mode.type <= SHAREBASIC) {
+	kill_on_powerfail = 0;
+	log_event(ups, LOG_WARNING,
+		  _("Ignoring --kill-on-powerfail since it is unsafe "
+		    "on Simple Signaling UPSes"));
+    }
+
     /*
      * Attach the correct driver.
      */
@@ -333,7 +275,7 @@ int main(int argc, char *argv[]) {
     ups->start_time = time(NULL);
     delete_lockfile(ups);
 
-    if (go_background) {
+    if (!kill_ups_power && go_background) {
        daemon_start();
        /* Reopen log file, closed during becoming daemon */
        openlog("apcupsd", LOG_CONS|LOG_PID, ups->sysfac);   
@@ -343,8 +285,8 @@ int main(int argc, char *argv[]) {
     make_pid_file();
     init_signals(apcupsd_terminate);
 
-    /* Create temp events file */
-    if (ups->eventfile[0] != 0) {
+    /* Create temp events file if we are not doing a killpower */
+    if (!kill_ups_power && ups->eventfile[0] != 0) {
 	ups->event_fd = open(ups->eventfile, O_RDWR|O_CREAT|O_APPEND, 0644);
 	if (ups->event_fd < 0) {
             log_event(ups, LOG_WARNING, "Could not open events file %s: %s\n",
@@ -468,6 +410,8 @@ int main(int argc, char *argv[]) {
     return 0;				/* to keep compiler happy */
 }
 
+extern int debug_level;
+
 /*
  * Initialize a daemon process completely detaching us from
  * any terminal processes.
@@ -475,39 +419,60 @@ int main(int argc, char *argv[]) {
 static void daemon_start(void)
 {
 #ifndef HAVE_CYGWIN
-    int i;
-    pid_t cpid;
-    mode_t oldmask;
+   int i, fd;
+   pid_t cpid;
+   mode_t oldmask;
+   int low_fd = -1;
 
-    if ( (cpid = fork() ) < 0) {
-        Error_abort0("Cannot fork to become daemon\n");
-    } else if (cpid > 0) {
-	exit(0);		  /* parent exits */
-    }
+   if ( (cpid = fork() ) < 0) {
+       Error_abort0("Cannot fork to become daemon\n");
+   } else if (cpid > 0) {
+       exit(0); 		 /* parent exits */
+   }
 
-    /* Child continues */
-    setsid();			       /* become session leader */
+   /* Child continues */
+   setsid();			      /* become session leader */
 
-    /* Call closelog() to close syslog file descriptor */
-    closelog();
+   /* Call closelog() to close syslog file descriptor */
+   closelog();
+
+   /* In the PRODUCTION system, we close ALL file descriptors unless
+    *  debugging is on then we laeve stdin, stdout, and stderr open.
+    */
+   if (debug_level > 0) {
+      low_fd = 2;                     /* don't close debug output */
+   }
+   for (i=sysconf(_SC_OPEN_MAX)-1; i > low_fd; i--) {
+      close(i);
+   }
+
+   /* move to root directory */
+   chdir("/");
+  
+   /* 
+    * Avoid creating files 0666 but don't override a
+    * more restrictive umask set by the caller.
+    */
+   oldmask = umask(022);
+   oldmask |= 022;
+   umask(oldmask);
+
+   /*
+    * Make sure we have fd's 0, 1, 2 open
+    *  If we don't do this one of our sockets may open
+    *  there and if we then use stdout, it could
+    *  send total garbage to our socket.
+    *
+    */
+   fd = open("/dev/null", O_RDONLY, 0644);
+   if (fd > 2) {
+      close(fd);
+   } else {
+      for(i=1; fd + i <= 2; i++) {
+	 dup2(fd, fd+i);
+      }
+   }
 
-    /* In the PRODUCTION system, we close ALL
-     * file descriptors except stdin, stdout, and stderr.
-     */
-    for (i=sysconf(_SC_OPEN_MAX)-1; i > 2; i--) {
-       close(i);
-    }
-											
-    /* move to root directory */
-    chdir("/");
-   
-    /* 
-     * Avoid creating files 0666 but don't override a
-     * more restrictive umask set by the caller.
-     */
-    oldmask = umask(022);
-    oldmask |= 022;
-    umask(oldmask);
 
 #endif /* HAVE_CYGWIN */
 }
