@@ -1,59 +1,31 @@
 /*
  *  apcstatus.c  -- Output STATUS information
  *
- *   Kern Sibbald 14 November 1999
- *   Copyright (C) 1999 Kern Sibbald
+ *   Written by Kern Sibbald 14 November 1999
  *
  *  apcupsd.c -- Simple Daemon to catch power failure signals from a
  *		 BackUPS, BackUPS Pro, or SmartUPS (from APCC).
  *	      -- Now SmartMode support for SmartUPS and BackUPS Pro.
- *
- *  Copyright (C) 1996-99 Andre M. Hedrick <andre@suse.com>
- *  All rights reserved.
- *
- *		       GNU GENERAL PUBLIC LICENSE
- *			  Version 2, June 1991
- *
- *  Copyright (C) 1989, 1991 Free Software Foundation, Inc.
- *			     675 Mass Ave, Cambridge, MA 02139, USA
- *  Everyone is permitted to copy and distribute verbatim copies
- *  of this license document, but changing it is not allowed.
- *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
- *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
- *
  */
 
 /*
- *  IN NO EVENT SHALL ANY AND ALL PERSONS INVOLVED IN THE DEVELOPMENT OF THIS
- *  PACKAGE, NOW REFERRED TO AS "APCUPSD-Team" BE LIABLE TO ANY PARTY FOR 
- *  DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING
- *  OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF ANY OR ALL
- *  OF THE "APCUPSD-Team" HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
- *
- *  THE "APCUPSD-Team" SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING,
- *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
- *  FITNESS FOR A PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS
- *  ON AN "AS IS" BASIS, AND THE "APCUPSD-Team" HAS NO OBLIGATION TO PROVIDE
- *  MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
- *
- *  THE "APCUPSD-Team" HAS ABSOLUTELY NO CONNECTION WITH THE COMPANY
- *  AMERICAN POWER CONVERSION, "APCC".  THE "APCUPSD-Team" DID NOT AND
- *  HAS NOT SIGNED ANY NON-DISCLOSURE AGREEMENTS WITH "APCC".  ANY AND ALL
- *  OF THE LOOK-A-LIKE ( UPSlink(tm) Language ) WAS DERIVED FROM THE
- *  SOURCES LISTED BELOW.
- *
+   Copyright (C) 1999-2004 Kern Sibbald
+
+   This program is free software; you can redistribute it and/or
+   modify it under the terms of the GNU General Public License as
+   published by the Free Software Foundation; either version 2 of
+   the License, or (at your option) any later version.
+
+   This program is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
+   General Public License for more details.
+
+   You should have received a copy of the GNU General Public
+   License along with this program; if not, write to the Free
+   Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
+   MA 02111-1307, USA.
+
  */
 
 #include "apc.h"
@@ -76,9 +48,11 @@ int output_status(UPSINFO *ups, int sockfd,
 
     s_open(ups);
 
+#ifdef HAVE_PTHREADS
     if (ups->poll_time == 0) {	  /* this is always zero on slave */
        ups->poll_time = now;
     }
+#endif
 
     localtime_r(&ups->poll_time, &tm);
     strftime(datetime, 100, "%a %b %d %X %Z %Y", &tm);
@@ -181,30 +155,30 @@ int output_status(UPSINFO *ups, int sockfd,
 	status[0] = 0;
 	/* Now output human readable form */
 	if (is_ups_set(UPS_CALIBRATION))
-            strcat(status, "CAL ");
+            astrncat(status, "CAL ", sizeof(status));
 	if (is_ups_set(UPS_SMARTTRIM))
-            strcat(status, "TRIM ");
+            astrncat(status, "TRIM ", sizeof(status));
 	if (is_ups_set(UPS_SMARTBOOST))
-            strcat(status, "BOOST ");
+            astrncat(status, "BOOST ", sizeof(status));
 	if (is_ups_set(UPS_ONLINE))
-            strcat(status, "ONLINE ");
+            astrncat(status, "ONLINE ", sizeof(status));
 	if (is_ups_set(UPS_ONBATT))
-            strcat(status, "ONBATT ");
+            astrncat(status, "ONBATT ", sizeof(status));
 	if (is_ups_set(UPS_OVERLOAD))
-            strcat(status, "OVERLOAD ");
+            astrncat(status, "OVERLOAD ", sizeof(status));
 	if (is_ups_set(UPS_BATTLOW))
-            strcat(status, "LOWBATT ");
+            astrncat(status, "LOWBATT ", sizeof(status));
 	if (is_ups_set(UPS_REPLACEBATT))
-            strcat(status, "REPLACEBATT ");
+            astrncat(status, "REPLACEBATT ", sizeof(status));
 	if (is_ups_set(UPS_SLAVE))
-            strcat(status, "SLAVE ");
+            astrncat(status, "SLAVE ", sizeof(status));
 	if (is_ups_set(UPS_SLAVEDOWN))
-            strcat(status, "SLAVEDOWN");
+            astrncat(status, "SLAVEDOWN", sizeof(status));
 	/* These override the above */
 	if (is_ups_set(UPS_COMMLOST))
-            strcpy(status, "COMMLOST ");
+            astrncpy(status, "COMMLOST ", sizeof(status));
 	if (is_ups_set(UPS_SHUTDOWN))
-            strcpy(status, "SHUTTING DOWN");
+            astrncpy(status, "SHUTTING DOWN", sizeof(status));
         s_write(ups, "STATUS   : %s\n", status);
 
 	if (ups->UPS_Cap[CI_VLINE]) {
@@ -247,9 +221,11 @@ int output_status(UPSINFO *ups, int sockfd,
                 s_write(ups, "SENSE    : Medium\n");
 		break;
             case 'H':
-	    default :
                 s_write(ups, "SENSE    : High\n");
 		break;
+	    default :
+                s_write(ups, "SENSE    : Unknown\n");
+		break;
 	    }
 	}
 
@@ -322,6 +298,9 @@ int output_status(UPSINFO *ups, int sockfd,
             case 'R':
                 s_write(ups, "LASTXFER : Unacceptable line voltage changes\n");
 		break;
+            case 'K':
+                s_write(ups, "LASTXFER : Software command or UPS's test control\n");
+		break;
 	    default :
                 s_write(ups, "LASTXFER : UNKNOWN EVENT %s\n", ups->G);
 		break;
@@ -509,21 +488,21 @@ char *ups_status(int stat)
 	buf[0] = 0;
 	/* Now output human readable form */
 	if (is_ups_set(UPS_CALIBRATION))
-            strcat(buf, "CAL ");
+            astrncat(buf, "CAL ", sizeof(buf));
 	if (is_ups_set(UPS_SMARTTRIM))
-            strcat(buf, "TRIM ");
+            astrncat(buf, "TRIM ", sizeof(buf));
 	if (is_ups_set(UPS_SMARTBOOST))
-            strcat(buf, "BOOST ");
+            astrncat(buf, "BOOST ", sizeof(buf));
 	if (is_ups_set(UPS_ONLINE))
-            strcat(buf, "ONLINE ");
+            astrncat(buf, "ONLINE ", sizeof(buf));
 	if (is_ups_set(UPS_ONBATT))
-            strcat(buf, "ON BATTERY ");
+            astrncat(buf, "ON BATTERY ", sizeof(buf));
 	if (is_ups_set(UPS_OVERLOAD))
-            strcat(buf, "OVERLOAD ");
+            astrncat(buf, "OVERLOAD ", sizeof(buf));
 	if (is_ups_set(UPS_BATTLOW))
-            strcat(buf, "LOWBATT ");
+            astrncat(buf, "LOWBATT ", sizeof(buf));
 	if (is_ups_set(UPS_REPLACEBATT))
-            strcat(buf, "REPLACEBATT ");
+            astrncat(buf, "REPLACEBATT ", sizeof(buf));
 	if (!is_ups_set(UPS_ONBATT) && ups->UPS_Cap[CI_BATTLEV])
 	    battstat = (int)ups->BattChg;
 	break;
