@@ -1,9 +1,12 @@
-//                              -*- Mode: C++ -*- 
+//                              -*- Mode: C++ -*-
 // compat.cpp -- compatibilty layer to make bacula-fd run
 //               natively under windows
 //
-// Copyright (C)2004 Raider Solutions, Inc
-// 
+// Copyright transferred from Raider Solutions, Inc to
+//   Kern Sibbald and John Walker by express permission.
+//
+// Copyright (C) 2004 Kern Sibbald and John Walker
+//
 //   This program is free software; you can redistribute it and/or
 //   modify it under the terms of the GNU General Public License as
 //   published by the Free Software Foundation; either version 2 of
@@ -21,23 +24,26 @@
 //
 // Author          : Christopher S. Hull
 // Created On      : Sat Jan 31 15:55:00 2004
-// Last Modified By: Christopher S. Hull
-// Last Modified On: Tue Feb 24 11:27:33 2004
-// Update Count    : 665
-// $Id: compat.cpp,v 1.1 2004/04/16 13:18:45 kerns Exp $
+// $Id: compat.cpp,v 1.2 2005/03/19 09:55:10 kerns Exp $
 
 #include <stdio.h>
 
 #include "compat.h"
 #include "pthread.h"
 
+#define b_errno_win32 (1<<29)
+
 #define USE_WIN32_COMPAT_IO 1
 
 extern void d_msg(const char *file, int line, int level, const char *fmt,...);
 extern DWORD   g_platform_id;
 
-// from CYGWIN (should be diff between Jan 1 1601 and Jan 1 1970
+// from MicroSoft SDK (KES) is the diff between Jan 1 1601 and Jan 1 1970
+#ifdef HAVE_MINGW
+#define WIN32_FILETIME_ADJUST 0x19DB1DED53E8000UL //Not sure it works
+#else
 #define WIN32_FILETIME_ADJUST 0x19DB1DED53E8000I64
+#endif
 
 #define WIN32_FILETIME_SCALE  10000000             // 100ns/second
 
@@ -92,30 +98,30 @@ wchar_win32_path(const char *name, WCHAR *win32_name)
     }
 }
 
-int
-umask(int)
+int umask(int)
 {
-    return 0;
+   return 0;
 }
 
-int
-chmod(const char *, mode_t)
+int chmod(const char *, mode_t)
 {
-    return 0;
+   return 0;
 }
 
-int
-chown(const char *k, uid_t, gid_t)
+int chown(const char *k, uid_t, gid_t)
 {
-    return 0;
+   return 0;
 }
 
-int
-lchown(const char *k, uid_t, gid_t)
+int lchown(const char *k, uid_t, gid_t)
 {
-    return 0;
+   return 0;
 }
 
+bool fstype(const char *fname, char *fs, int fslen)
+{
+   return true;                       /* accept anything */
+}
 
 
 long int
@@ -127,7 +133,7 @@ random(void)
 void
 srandom(unsigned int seed)
 {
-    srand(seed);
+   srand(seed);
 }
 // /////////////////////////////////////////////////////////////////
 // convert from Windows concept of time to Unix concept of time
@@ -139,7 +145,11 @@ cvt_utime_to_ftime(const time_t  &time, FILETIME &wintime)
     mstime *= WIN32_FILETIME_SCALE;
     mstime += WIN32_FILETIME_ADJUST;
 
+    #ifdef HAVE_MINGW
+    wintime.dwLowDateTime = (DWORD)(mstime & 0xffffffffUL);
+    #else
     wintime.dwLowDateTime = (DWORD)(mstime & 0xffffffffI64);
+    #endif
     wintime.dwHighDateTime = (DWORD) ((mstime>>32)& 0xffffffffUL);
 }
 
@@ -159,21 +169,23 @@ cvt_ftime_to_utime(const FILETIME &time)
 static const char *
 errorString(void)
 {
-    LPVOID lpMsgBuf;
+   LPVOID lpMsgBuf;
 
-    FormatMessage(FORMAT_MESSAGE_ALLOCATE_BUFFER |
-                  FORMAT_MESSAGE_FROM_SYSTEM |
-                  FORMAT_MESSAGE_IGNORE_INSERTS,
-                  NULL,
-                  GetLastError(),
-                  MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), // Default lang
-                  (LPTSTR) &lpMsgBuf,
-                  0,
-                  NULL);
+   FormatMessage(FORMAT_MESSAGE_ALLOCATE_BUFFER |
+                 FORMAT_MESSAGE_FROM_SYSTEM |
+                 FORMAT_MESSAGE_IGNORE_INSERTS,
+                 NULL,
+                 GetLastError(),
+                 MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), // Default lang
+                 (LPTSTR) &lpMsgBuf,
+                 0,
+                 NULL);
 
-    return (const char *) lpMsgBuf;
+   return (const char *) lpMsgBuf;
 }
 
+#ifndef HAVE_MINGW
+
 static int
 statDir(const char *file, struct stat *sb)
 {
@@ -194,7 +206,7 @@ statDir(const char *file, struct stat *sb)
         const char *err = errorString();
         d_msg(__FILE__, __LINE__, 99, "FindFirstFile(%s):%s\n", file, err);
         LocalFree((void *)err);
-        errno = GetLastError();
+        errno = b_errno_win32;
         return -1;
     }
 
@@ -215,7 +227,7 @@ statDir(const char *file, struct stat *sb)
 
     sb->st_atime = cvt_ftime_to_utime(info.ftLastAccessTime);
     sb->st_mtime = cvt_ftime_to_utime(info.ftLastWriteTime);
-    sb->st_ctime = cvt_ftime_to_utime(info.ftLastWriteTime);
+    sb->st_ctime = cvt_ftime_to_utime(info.ftCreationTime);
     FindClose(h);
 
     return 0;
@@ -237,11 +249,11 @@ stat2(const char *file, struct stat *sb)
         d_msg(__FILE__, __LINE__, 99,
               "GetFileAttrubtes(%s): %s\n", tmpbuf, err);
         LocalFree((void *)err);
-        errno = GetLastError();
+        errno = b_errno_win32;
         return -1;
     }
-    
-    if (attr & FILE_ATTRIBUTE_DIRECTORY) 
+
+    if (attr & FILE_ATTRIBUTE_DIRECTORY)
         return statDir(tmpbuf, sb);
 
     h = CreateFile(tmpbuf, GENERIC_READ,
@@ -253,25 +265,28 @@ stat2(const char *file, struct stat *sb)
               "Cannot open file for stat (%s):%s\n", tmpbuf, err);
         LocalFree((void *)err);
         rval = -1;
-        errno = GetLastError();
+        errno = b_errno_win32;
         goto error;
     }
-    
+
     if (!GetFileInformationByHandle(h, &info)) {
         const char *err = errorString();
         d_msg(__FILE__, __LINE__, 99,
               "GetfileInformationByHandle(%s): %s\n", tmpbuf, err);
         LocalFree((void *)err);
         rval = -1;
-        errno = GetLastError();
+        errno = b_errno_win32;
         goto error;
     }
-    
+
     sb->st_dev = info.dwVolumeSerialNumber;
     sb->st_ino = info.nFileIndexHigh;
     sb->st_ino <<= 32;
     sb->st_ino |= info.nFileIndexLow;
     sb->st_nlink = (short)info.nNumberOfLinks;
+    if (sb->st_nlink > 1) {
+       d_msg(__FILE__, __LINE__, 99,  "st_nlink=%d\n", sb->st_nlink);
+    }
 
     sb->st_mode = 0777;               /* start with everything */
     if (info.dwFileAttributes & FILE_ATTRIBUTE_READONLY)
@@ -289,7 +304,7 @@ stat2(const char *file, struct stat *sb)
     sb->st_blocks = (uint32_t)(sb->st_size + 4095)/4096;
     sb->st_atime = cvt_ftime_to_utime(info.ftLastAccessTime);
     sb->st_mtime = cvt_ftime_to_utime(info.ftLastWriteTime);
-    sb->st_ctime = cvt_ftime_to_utime(info.ftLastWriteTime);
+    sb->st_ctime = cvt_ftime_to_utime(info.ftCreationTime);
 
 error:
     CloseHandle(h);
@@ -324,7 +339,7 @@ stat(const char *file, struct stat *sb)
     if (!GetFileAttributesEx(file, GetFileExInfoStandard, &data))
         return stat2(file, sb);
 #endif
-    
+
     sb->st_mode = 0777;               /* start with everything */
     if (data.dwFileAttributes & FILE_ATTRIBUTE_READONLY)
         sb->st_mode &= ~(S_IRUSR|S_IRGRP|S_IROTH);
@@ -346,10 +361,12 @@ stat(const char *file, struct stat *sb)
     sb->st_blocks = (uint32_t)(sb->st_size + 4095)/4096;
     sb->st_atime = cvt_ftime_to_utime(data.ftLastAccessTime);
     sb->st_mtime = cvt_ftime_to_utime(data.ftLastWriteTime);
-    sb->st_ctime = cvt_ftime_to_utime(data.ftLastWriteTime);
+    sb->st_ctime = cvt_ftime_to_utime(data.ftCreationTime);
     return 0;
 }
 
+#endif //HAVE_MINGW
+
 int
 lstat(const char *file, struct stat *sb)
 {
@@ -360,7 +377,7 @@ void
 sleep(int sec)
 {
     Sleep(sec * 1000);
-}    
+}
 
 int
 geteuid(void)
@@ -370,6 +387,7 @@ geteuid(void)
 
 int
 execvp(const char *, char *[]) {
+    errno = ENOSYS;
     return -1;
 }
 
@@ -377,33 +395,38 @@ execvp(const char *, char *[]) {
 int
 fork(void)
 {
+    errno = ENOSYS;
     return -1;
 }
 
 int
 pipe(int[])
 {
+    errno = ENOSYS;
     return -1;
 }
 
 int
 waitpid(int, int*, int)
 {
+    errno = ENOSYS;
     return -1;
 }
 
 int
 readlink(const char *, char *, int)
 {
+    errno = ENOSYS;
     return -1;
 }
 
 
+#ifndef HAVE_MINGW
 int
 strcasecmp(const char *s1, const char *s2)
 {
     register int ch1, ch2;
-    
+
     if (s1==s2)
         return 0;       /* strings are equal if same object. */
     else if (!s1)
@@ -417,30 +440,32 @@ strcasecmp(const char *s1, const char *s2)
         s1++;
         s2++;
     } while (ch1 != 0 && tolower(ch1) == tolower(ch2));
-    
+
   return(ch1 - ch2);
 }
+#endif //HAVE_MINGW
 
 int
 strncasecmp(const char *s1, const char *s2, int len)
 {
     register int ch1, ch2;
-    
+
     if (s1==s2)
         return 0;       /* strings are equal if same object. */
     else if (!s1)
         return -1;
     else if (!s2)
         return 1;
-    do
+    while (len--)
     {
         ch1 = *s1;
         ch2 = *s2;
         s1++;
         s2++;
-    } while (len-- && ch1 != 0 && tolower(ch1) == tolower(ch2));
-    
-    return(ch1 - ch2);
+        if (ch1 == 0 || tolower(ch1) != tolower(ch2)) break;
+    } 
+
+    return (ch1 - ch2);
 }
 
 int
@@ -450,23 +475,32 @@ gettimeofday(struct timeval *tv, struct timezone *)
     FILETIME tmp;
     GetSystemTime(&now);
 
-    if (tv == NULL) return -1;
-    if (!SystemTimeToFileTime(&now, &tmp)) return -1;
+    if (tv == NULL) {
+       errno = EINVAL;
+       return -1;
+    }
+    if (!SystemTimeToFileTime(&now, &tmp)) {
+       errno = b_errno_win32;
+       return -1;
+    }
 
     int64_t _100nsec = tmp.dwHighDateTime;
     _100nsec <<= 32;
     _100nsec |= tmp.dwLowDateTime;
     _100nsec -= WIN32_FILETIME_ADJUST;
-    
+
     tv->tv_sec =(long) (_100nsec / 10000000);
     tv->tv_usec = (long) ((_100nsec % 10000000)/10);
     return 0;
-    
+
 }
 
 int
-syslog(int, const char *, const char *)
+syslog(int type, const char *fmt, const char *msg)
 {
+/*#ifndef HAVE_CONSOLE
+    MessageBox(NULL, msg, "Bacula", MB_OK);
+#endif*/
     return 0;
 }
 
@@ -482,7 +516,6 @@ getgrgid(uid_t)
     return NULL;
 }
 
-
 // implement opendir/readdir/closedir on top of window's API
 typedef struct _dir
 {
@@ -498,13 +531,16 @@ opendir(const char *path)
 {
     int max_len = strlen(path) + 16;
     _dir *rval = NULL;
-    if (path == NULL) return NULL;
-    
+    if (path == NULL) {
+       errno = ENOENT;
+       return NULL;
+    }
+
     rval = (_dir *)malloc(sizeof(_dir));
     if (rval == NULL) return NULL;
     char *tspec = (char *)malloc(max_len);
-    if (tspec == NULL) goto err1;
-    
+    if (tspec == NULL) return NULL;
+
     if (g_platform_id != VER_PLATFORM_WIN32_WINDOWS) {
         // allow path to be 32767 bytes
         tspec[0] = '\\';
@@ -512,10 +548,11 @@ opendir(const char *path)
         tspec[2] = '?';
         tspec[3] = '\\';
         tspec[4] = 0;
-        cygwin_conv_to_win32_path(path, tspec+4); 
+        cygwin_conv_to_win32_path(path, tspec+4);
     } else {
         cygwin_conv_to_win32_path(path, tspec);
     }
+
     strncat(tspec, "\\*", max_len);
     rval->spec = tspec;
 
@@ -531,13 +568,14 @@ opendir(const char *path)
     d_msg(__FILE__, __LINE__,
           99, "\tFirstFile=%s\n", rval->data.cFileName);
     return (DIR *)rval;
+
 err:
     free((void *)rval->spec);
-err1:
     free(rval);
+    errno = b_errno_win32;
     return NULL;
 }
-    
+
 int
 closedir(DIR *dirp)
 {
@@ -576,10 +614,10 @@ copyin(struct dirent &dp, const char *fname)
         *cp = 0;
     return dp.d_reclen;
 }
+
 int
 readdir_r(DIR *dirp, struct dirent *entry, struct dirent **result)
 {
-
     _dir *dp = (_dir *)dirp;
     if (dp->valid) {
         entry->d_off = dp->offset;
@@ -590,35 +628,48 @@ readdir_r(DIR *dirp, struct dirent *entry, struct dirent **result)
               dirp, entry->d_name, entry->d_reclen, entry->d_off);
     } else {
 //      d_msg(__FILE__, __LINE__, 99, "readdir_r !valid\n");
+        errno = b_errno_win32;
         return -1;
     }
     dp->valid = FindNextFile(dp->dirh, &dp->data);
     return 0;
 }
 
-
+/*
+ * Dotted IP address to network address
+ *
+ * Returns 1 if  OK
+ *         0 on error
+ */
 int
 inet_aton(const char *a, struct in_addr *inp)
 {
+   const char *cp = a;
+   uint32_t acc = 0, tmp = 0;
+   int dotc = 0;
 
-    const char *cp = a;
-    uint32_t acc = 0, tmp = 0;
-    int dotc = 0;
-    if (!isdigit(*a)) return 0;
-    while (*cp) {
-        if (isdigit(*cp))
-            tmp = (tmp * 10) + (*cp -'0');
-        else if (*cp == '.') {
-            if (tmp > 255) return 0;
-            acc = (acc << 8) + tmp;
-            dotc++;
-        }
-        else return 0;
-    }
-
-    if (dotc != 3) return 0;
-    inp->s_addr = acc;
-    return 1;
+   if (!isdigit(*cp)) {         /* first char must be digit */
+      return 0;                 /* error */
+   }
+   do {
+      if (isdigit(*cp)) {
+         tmp = (tmp * 10) + (*cp -'0');
+      } else if (*cp == '.' || *cp == 0) {
+         if (tmp > 255) {
+            return 0;           /* error */
+         }
+         acc = (acc << 8) + tmp;
+         dotc++;
+         tmp = 0;
+      } else {
+         return 0;              /* error */
+      }
+   } while (*cp++ != 0);
+   if (dotc != 4) {              /* want 3 .'s plus EOS */
+      return 0;                  /* error */
+   }
+   inp->s_addr = htonl(acc);     /* store addr in network format */
+   return 1;
 }
 
 int
@@ -653,7 +704,7 @@ pathconf(const char *path, int name)
     case _PC_NAME_MAX :
         return 255;
     }
-
+    errno = ENOSYS;                                                         
     return -1;
 }
 
@@ -662,13 +713,13 @@ WSA_Init(void)
 {
     WORD wVersionRequested = MAKEWORD( 1, 1);
     WSADATA wsaData;
-    
+
     int err = WSAStartup(wVersionRequested, &wsaData);
-    
-    
-    if (err != 0)
-    {
+
+
+    if (err != 0) {
         printf("Can not start Windows Sockets\n");
+        errno = ENOSYS;
         return -1;
     }
 
@@ -679,7 +730,10 @@ WSA_Init(void)
 int
 win32_chdir(const char *dir)
 {
-    if (0 == SetCurrentDirectory(dir)) return -1;
+    if (0 == SetCurrentDirectory(dir)) {
+       errno = b_errno_win32;
+       return -1;
+    }
     return 0;
 }
 
@@ -695,7 +749,6 @@ win32_getcwd(char *buf, int maxlen)
        buf[n] = '\\';
        buf[n+1] = 0;
    }
-   
    return buf;
 }
 
@@ -743,7 +796,7 @@ winver::winver(void)
     snprintf(WIN_VERSION, sizeof(WIN_VERSION), "%s %d.%d.%d",
              platform, osvinfo.dwMajorVersion, osvinfo.dwMinorVersion, osvinfo.dwBuildNumber);
 
-#if 1
+#if 0
     HANDLE h = CreateFile("G:\\foobar", GENERIC_WRITE, 0, NULL, CREATE_ALWAYS, 0, NULL);
     CloseHandle(h);
 #endif
@@ -756,26 +809,40 @@ winver::winver(void)
     close_bpipe(b);
 #endif
 }
- 
-BOOL CreateChildProcess(VOID); 
-VOID WriteToPipe(VOID); 
-VOID ReadFromPipe(VOID); 
-VOID ErrorExit(LPTSTR); 
-VOID ErrMsg(LPTSTR, BOOL); 
- 
 
+BOOL CreateChildProcess(VOID);
+VOID WriteToPipe(VOID);
+VOID ReadFromPipe(VOID);
+VOID ErrorExit(LPTSTR);
+VOID ErrMsg(LPTSTR, BOOL);
+
+/**
+ * Check for a quoted path,  if an absolute path name is given and it contains
+ * spaces it will need to be quoted.  i.e.  "c:/Program Files/foo/bar.exe"
+ * CreateProcess() says the best way to ensure proper results with executables
+ * with spaces in path or filename is to quote the string.
+ */
 const char *
 getArgv0(const char *cmdline)
 {
-    const char *cp = cmdline;
 
-    while (*cp && !isspace(*cp)) cp++;
+    int inquote = 0;
+    for (const char *cp = cmdline; *cp; cp++)
+    {
+        if (*cp == '"') {
+            inquote = !inquote;
+        }
+        if (!inquote && isspace(*cp))
+            break;
+    }
 
+        
     int len = cp - cmdline;
     char *rval = (char *)malloc(len+1);
 
     cp = cmdline;
     char *rp = rval;
+    
     while (len--)
         *rp++ = *cp++;
 
@@ -783,19 +850,26 @@ getArgv0(const char *cmdline)
     return rval;
 }
 
+
+/**
+ * OK, so it would seem CreateProcess only handles true executables:
+ *  .com or .exe files.
+ * So test to see whether we're getting a .bat file and if so grab
+ * $COMSPEC value and pass batch file to it.
+ */
 HANDLE
-CreateChildProcess(const char *cmdline, HANDLE in, HANDLE out, HANDLE err) 
-{ 
-    PROCESS_INFORMATION piProcInfo; 
+CreateChildProcess(const char *cmdline, HANDLE in, HANDLE out, HANDLE err)
+{
+    PROCESS_INFORMATION piProcInfo;
     STARTUPINFO siStartInfo;
-    BOOL bFuncRetn = FALSE; 
-    
-    // Set up members of the PROCESS_INFORMATION structure. 
-    
+    BOOL bFuncRetn = FALSE;
+
+    // Set up members of the PROCESS_INFORMATION structure.
+
     ZeroMemory( &piProcInfo, sizeof(PROCESS_INFORMATION) );
-    
-    // Set up members of the STARTUPINFO structure. 
-    
+
+    // Set up members of the STARTUPINFO structure.
+
     ZeroMemory( &siStartInfo, sizeof(STARTUPINFO) );
     siStartInfo.cb = sizeof(STARTUPINFO);
     // setup new process to use supplied handles for stdin,stdout,stderr
@@ -807,7 +881,7 @@ CreateChildProcess(const char *cmdline, HANDLE in, HANDLE out, HANDLE err)
         siStartInfo.hStdInput = in;
     else
         siStartInfo.hStdInput = GetStdHandle(STD_INPUT_HANDLE);
-    
+
     if (out != INVALID_HANDLE_VALUE)
         siStartInfo.hStdOutput = out;
     else
@@ -816,69 +890,56 @@ CreateChildProcess(const char *cmdline, HANDLE in, HANDLE out, HANDLE err)
         siStartInfo.hStdError = err;
     else
         siStartInfo.hStdError = GetStdHandle(STD_ERROR_HANDLE);
-    // Create the child process. 
+    // Create the child process.
+
+    char exeFile[256];
+
+    const char *comspec = getenv("COMSPEC");
     
-    char cmdLine[1024];
-    char exeFile[1024];
-    // retrive the first compont of the command line which should be the
-    // executable 
-    const char *exeName = getArgv0(cmdline);
-    // check to see if absolute path was passed to us already?
-    if (exeName[1] != ':'
-        || (strchr(cmdline, '/') == NULL
-            && strchr(cmdline, '\\') == NULL))
-    {
-        // only command name so perform search of PATH to find 
-        char *file;
-        DWORD rval = SearchPath(NULL,
-                                exeName,
-                                ".exe",
-                                sizeof(exeFile),
-                                exeFile,
-                                &file);
-        if (rval == 0)
-            return INVALID_HANDLE_VALUE;
-        if (rval > sizeof(exeFile))
-            return INVALID_HANDLE_VALUE;
+    if (comspec == NULL) // should never happen
+        return INVALID_HANDLE_VALUE;
 
-    }
-    else 
-        strcpy(exeFile, exeName);
+    char *cmdLine = (char *)alloca(strlen(cmdline) + strlen(comspec) + 16);
+    
+    strcpy(exeFile, comspec);
+    strcpy(cmdLine, comspec);
+    strcat(cmdLine, " /c ");
+    strcat(cmdLine, cmdline);
 
-    // exeFile now has absolute path to program to execute.
-    free((void *)exeName);
-    // copy original command line to pass to create process
-    strcpy(cmdLine, cmdline);
     // try to execute program
-    bFuncRetn = CreateProcess(exeFile, 
-                              cmdLine, // command line 
-                              NULL, // process security attributes 
-                              NULL, // primary thread security attributes 
-                              TRUE, // handles are inherited 
-                              0, // creation flags 
-                              NULL, // use parent's environment 
-                              NULL, // use parent's current directory 
-                              &siStartInfo, // STARTUPINFO pointer 
+    bFuncRetn = CreateProcess(exeFile,
+                              cmdLine, // command line
+                              NULL, // process security attributes
+                              NULL, // primary thread security attributes
+                              TRUE, // handles are inherited
+                              0, // creation flags
+                              NULL, // use parent's environment
+                              NULL, // use parent's current directory
+                              &siStartInfo, // STARTUPINFO pointer
                               &piProcInfo); // receives PROCESS_INFORMATION
-    
+
     if (bFuncRetn == 0) {
         ErrorExit("CreateProcess failed\n");
+        const char *err = errorString();
+        d_msg(__FILE__, __LINE__, 99,
+              "CreateProcess(%s, %s, ...)=%s\n", exeFile, cmdLine, err);
+        LocalFree((void *)err);
         return INVALID_HANDLE_VALUE;
     }
     // we don't need a handle on the process primary thread so we close
     // this now.
     CloseHandle(piProcInfo.hThread);
-    
+
     return piProcInfo.hProcess;
 }
 
- 
+
 void
-ErrorExit (LPTSTR lpszMessage) 
-{ 
+ErrorExit (LPTSTR lpszMessage)
+{
     d_msg(__FILE__, __LINE__, 0, "%s", lpszMessage);
-} 
- 
+}
+
 
 /*
 typedef struct s_bpipe {
@@ -898,44 +959,45 @@ CloseIfValid(HANDLE handle)
         CloseHandle(handle);
 }
 
+#ifndef HAVE_MINGW
 BPIPE *
 open_bpipe(char *prog, int wait, const char *mode)
 {
-    HANDLE hChildStdinRd, hChildStdinWr, hChildStdinWrDup, 
-        hChildStdoutRd, hChildStdoutWr, hChildStdoutRdDup, 
+    HANDLE hChildStdinRd, hChildStdinWr, hChildStdinWrDup,
+        hChildStdoutRd, hChildStdoutWr, hChildStdoutRdDup,
         hInputFile;
-    
-    SECURITY_ATTRIBUTES saAttr; 
 
-    BOOL fSuccess; 
+    SECURITY_ATTRIBUTES saAttr;
 
-    hChildStdinRd = hChildStdinWr = hChildStdinWrDup = 
-        hChildStdoutRd = hChildStdoutWr = hChildStdoutRdDup = 
+    BOOL fSuccess;
+
+    hChildStdinRd = hChildStdinWr = hChildStdinWrDup =
+        hChildStdoutRd = hChildStdoutWr = hChildStdoutRdDup =
         hInputFile = INVALID_HANDLE_VALUE;
-    
+
     BPIPE *bpipe = (BPIPE *)malloc(sizeof(BPIPE));
     memset((void *)bpipe, 0, sizeof(BPIPE));
 
     int mode_read = (mode[0] == 'r');
     int mode_write = (mode[0] == 'w' || mode[1] == 'w');
-    
-    
-    // Set the bInheritHandle flag so pipe handles are inherited. 
-    
-    saAttr.nLength = sizeof(SECURITY_ATTRIBUTES); 
-    saAttr.bInheritHandle = TRUE; 
-    saAttr.lpSecurityDescriptor = NULL; 
-    
+
+
+    // Set the bInheritHandle flag so pipe handles are inherited.
+
+    saAttr.nLength = sizeof(SECURITY_ATTRIBUTES);
+    saAttr.bInheritHandle = TRUE;
+    saAttr.lpSecurityDescriptor = NULL;
+
     if (mode_read) {
-        
-        // Create a pipe for the child process's STDOUT. 
+
+        // Create a pipe for the child process's STDOUT.
         if (! CreatePipe(&hChildStdoutRd, &hChildStdoutWr, &saAttr, 0)) {
             ErrorExit("Stdout pipe creation failed\n");
             goto cleanup;
         }
-        // Create noninheritable read handle and close the inheritable read 
-        // handle. 
-        
+        // Create noninheritable read handle and close the inheritable read
+        // handle.
+
         fSuccess = DuplicateHandle(GetCurrentProcess(), hChildStdoutRd,
                                    GetCurrentProcess(), &hChildStdoutRdDup , 0,
                                    FALSE,
@@ -944,136 +1006,155 @@ open_bpipe(char *prog, int wait, const char *mode)
             ErrorExit("DuplicateHandle failed");
             goto cleanup;
         }
-        
+
         CloseHandle(hChildStdoutRd);
+        hChildStdoutRd = INVALID_HANDLE_VALUE;
     }
-    
+
     if (mode_write) {
-        
-        // Create a pipe for the child process's STDIN. 
-        
-        if (! CreatePipe(&hChildStdinRd, &hChildStdinWr, &saAttr, 0)) {
+
+        // Create a pipe for the child process's STDIN.
+
+        if (!CreatePipe(&hChildStdinRd, &hChildStdinWr, &saAttr, 0)) {
             ErrorExit("Stdin pipe creation failed\n");
             goto cleanup;
         }
-        
-        // Duplicate the write handle to the pipe so it is not inherited. 
-        fSuccess = DuplicateHandle(GetCurrentProcess(), hChildStdinWr, 
+
+        // Duplicate the write handle to the pipe so it is not inherited.
+        fSuccess = DuplicateHandle(GetCurrentProcess(), hChildStdinWr,
                                    GetCurrentProcess(), &hChildStdinWrDup,
-                                   0, 
-                                   FALSE,                  // not inherited 
-                                   DUPLICATE_SAME_ACCESS); 
+                                   0,
+                                   FALSE,                  // not inherited
+                                   DUPLICATE_SAME_ACCESS);
         if (!fSuccess) {
             ErrorExit("DuplicateHandle failed");
             goto cleanup;
         }
-        
-        CloseHandle(hChildStdinWr); 
+
+        CloseHandle(hChildStdinWr);
+        hChildStdinWr = INVALID_HANDLE_VALUE;
     }
     // spawn program with redirected handles as appropriate
     bpipe->worker_pid = (pid_t)
-        CreateChildProcess(prog, // commandline
-                           hChildStdinRd, // stdin HANDLE
-                           hChildStdoutWr, // stdout HANDLE
-                           hChildStdoutWr);// stderr HANDLE
-    
+        CreateChildProcess(prog,             // commandline
+                           hChildStdinRd,    // stdin HANDLE
+                           hChildStdoutWr,   // stdout HANDLE
+                           hChildStdoutWr);  // stderr HANDLE
+
     if ((HANDLE) bpipe->worker_pid == INVALID_HANDLE_VALUE)
         goto cleanup;
-    
+
     bpipe->wait = wait;
     bpipe->worker_stime = time(NULL);
-    
+
     if (mode_read) {
         CloseHandle(hChildStdoutWr); // close our write side so when
                                      // process terminates we can
                                      // detect eof.
         // ugly but convert WIN32 HANDLE to FILE*
         int rfd = _open_osfhandle((long)hChildStdoutRdDup, O_RDONLY);
-        bpipe->rfd = _fdopen(rfd, "r");
+        if (rfd >= 0) {
+           bpipe->rfd = _fdopen(rfd, "r");
+        }
     }
     if (mode_write) {
         CloseHandle(hChildStdinRd); // close our read side so as not
                                     // to interfre with child's copy
         // ugly but convert WIN32 HANDLE to FILE*
         int wfd = _open_osfhandle((long)hChildStdinWrDup, O_WRONLY);
-        bpipe->wfd = _fdopen(wfd, "w");
+        if (wfd >= 0) {
+           bpipe->wfd = _fdopen(wfd, "w");
+        }
     }
-    
+
     if (wait > 0) {
         bpipe->timer_id = start_child_timer(bpipe->worker_pid, wait);
     }
-    
+
     return bpipe;
-    
+
 cleanup:
-    
+
     CloseIfValid(hChildStdoutRd);
     CloseIfValid(hChildStdoutRdDup);
     CloseIfValid(hChildStdinWr);
     CloseIfValid(hChildStdinWrDup);
-    
+
     free((void *) bpipe);
-    
+    errno = b_errno_win32;            /* do GetLastError() for error code */
     return NULL;
 }
 
+#endif //HAVE_MINGW
+
 int
 kill(int pid, int signal)
 {
-    int rval = 0;
-    if (!TerminateProcess((HANDLE)pid, (UINT) signal))
-        rval = -1;
-    CloseHandle((HANDLE)pid);
-    return rval;
+   int rval = 0;
+   if (!TerminateProcess((HANDLE)pid, (UINT) signal)) {
+      rval = -1;
+      errno = b_errno_win32;
+   }
+   CloseHandle((HANDLE)pid);
+   return rval;
 }
 
+#ifndef HAVE_MINGW
+
 int
 close_bpipe(BPIPE *bpipe)
 {
-    int rval = 0;
-    if (bpipe->rfd) fclose(bpipe->rfd);
-    if (bpipe->wfd) fclose(bpipe->wfd);
+   int rval = 0;
+   int32_t remaining_wait = bpipe->wait;
 
-    if (bpipe->wait) {
-        int remaining_wait = bpipe->wait;
-        do 
-        {
-            DWORD exitCode;
-            if (!GetExitCodeProcess((HANDLE)bpipe->worker_pid, &exitCode))
-            {
-                const char *err = errorString();
-                rval = GetLastError();
-                d_msg(__FILE__, __LINE__, 0, 
-                      "GetExitCode error %s\n", err);
-                LocalFree((void *)err);
-                break;
-            }
-            
-            if (exitCode == STILL_ACTIVE) {
-                bmicrosleep(1, 0);             /* wait one second */
-                remaining_wait--;
-            }
-            else break;
-        } while(remaining_wait);
-    }
-    
-    if (bpipe->timer_id) {
-        stop_child_timer(bpipe->timer_id);
-    }
-    free((void *)bpipe);    
-    return rval;
+   if (remaining_wait == 0) {         /* wait indefinitely */
+      remaining_wait = INT32_MAX;
+   }
+   for ( ;; ) {
+      DWORD exitCode;
+      if (!GetExitCodeProcess((HANDLE)bpipe->worker_pid, &exitCode)) {
+         const char *err = errorString();
+         rval = b_errno_win32;
+         d_msg(__FILE__, __LINE__, 0,
+               "GetExitCode error %s\n", err);
+         LocalFree((void *)err);
+         break;
+      }
+      if (exitCode == STILL_ACTIVE) {
+         if (remaining_wait <= 0) {
+            rval = ETIME;             /* timed out */
+            break;
+         }
+         bmicrosleep(1, 0);           /* wait one second */
+         remaining_wait--;
+      } else if (exitCode != 0) {
+         /* Truncate exit code as it doesn't seem to be correct */
+         rval = (exitCode & 0xFF) | b_errno_exit;
+         break;
+      } else {
+         break;                       /* Shouldn't get here */
+      }
+   }
+
+   if (bpipe->timer_id) {
+       stop_child_timer(bpipe->timer_id);
+   }
+   if (bpipe->rfd) fclose(bpipe->rfd);
+   if (bpipe->wfd) fclose(bpipe->wfd);
+   free((void *)bpipe);
+   return rval;
 }
 
 int
 close_wpipe(BPIPE *bpipe)
 {
     int stat = 1;
-    
+
     if (bpipe->wfd) {
         fflush(bpipe->wfd);
         if (fclose(bpipe->wfd) != 0) {
             stat = 0;
-      }
+        }
         bpipe->wfd = NULL;
     }
     return stat;
@@ -1103,15 +1184,17 @@ utime(const char *fname, struct utimbuf *times)
     if (h == INVALID_HANDLE_VALUE) {
         const char *err = errorString();
         d_msg(__FILE__, __LINE__, 99,
-              "Cannot open file for utime(%s,...):%s\n", tmpbuf, err);
+              "Cannot open file \"%s\" for utime(): ERR=%s", tmpbuf, err);
         LocalFree((void *)err);
+        errno = b_errno_win32;
         return -1;
     }
 
     int rval = SetFileTime(h, NULL, &acc, &mod) ? 0 : -1;
-
     CloseHandle(h);
-    
+    if (rval == -1) {
+       errno = b_errno_win32;
+    }
     return rval;
 }
 
@@ -1124,23 +1207,31 @@ open(const char *file, int flags, int mode)
 
 }
 
+/*
+ * Note, this works only for a file. If you want
+ *   to close a socket, use closesocket(). 
+ *   Bacula has been modified in src/lib/bnet.c
+ *   to use closesocket().
+ */
 int
 close(int fd)
 {
     return _close(fd);
 }
 
+#ifndef HAVE_WXCONSOLE
 ssize_t
-read(int fd, void *buf, size_t len)
+read(int fd, void *buf, ssize_t len)
 {
-    return _read(fd, buf, len);
+    return _read(fd, buf, (size_t)len);
 }
 
 ssize_t
-write(int fd, const void *buf, size_t len)
+write(int fd, const void *buf, ssize_t len)
 {
-    return _write(fd, buf, len);
+    return _write(fd, buf, (size_t)len);
 }
+#endif
 
 off_t
 lseek(int fd, off_t offset, int whence)
@@ -1167,25 +1258,25 @@ open(const char *file, int flags, int mode)
     if (flags & O_WRONLY) access = GENERIC_WRITE;
     else if (flags & O_RDWR) access = GENERIC_READ|GENERIC_WRITE;
     else access = GENERIC_READ;
-    
+
     if (flags & O_CREAT) create = CREATE_NEW;
     else create = OPEN_EXISTING;
-    
+
     if (flags & O_TRUNC) create = TRUNCATE_EXISTING;
-    
+
     if (!(flags & O_EXCL))
         shareMode = FILE_SHARE_DELETE|FILE_SHARE_READ|FILE_SHARE_WRITE;
-    
-    if (flags & O_APPEND)
-    {
+
+    if (flags & O_APPEND) {
         printf("open...APPEND not implemented yet.");
         exit(-1);
     }
-    
-    foo = CreateFile(file, access, shareMode, NULL, create, msflags, NULL);
-    if (INVALID_HANDLE_VALUE == foo)
-        return(int) -1;
 
+    foo = CreateFile(file, access, shareMode, NULL, create, msflags, NULL);
+    if (INVALID_HANDLE_VALUE == foo) {
+        errno = b_errno_win32;
+        return(int) -1;
+    }
     return (int)foo;
 
 }
@@ -1194,31 +1285,35 @@ open(const char *file, int flags, int mode)
 int
 close(int fd)
 {
-    if (!CloseHandle((HANDLE)fd))
+    if (!CloseHandle((HANDLE)fd)) {
+        errno = b_errno_win32;
         return -1;
+    }
 
     return 0;
 }
 
 ssize_t
-write(int fd, const void *data, size_t len)
+write(int fd, const void *data, ssize_t len)
 {
     BOOL status;
     DWORD bwrite;
     status = WriteFile((HANDLE)fd, data, len, &bwrite, NULL);
     if (status) return bwrite;
+    errno = b_errno_win32;
     return -1;
 }
 
 
 ssize_t
-read(int fd, void *data, size_t len)
+read(int fd, void *data, ssize_t len)
 {
     BOOL status;
     DWORD bread;
 
     status = ReadFile((HANDLE)fd, data, len, &bread, NULL);
     if (status) return bread;
+    errno = b_errno_win32;
     return -1;
 }
 
@@ -1226,6 +1321,7 @@ off_t
 lseek(int fd, off_t offset, int whence)
 {
     DWORD method = 0;
+    DWORD val;
     switch (whence) {
     case SEEK_SET :
         method = FILE_BEGIN;
@@ -1237,17 +1333,31 @@ lseek(int fd, off_t offset, int whence)
         method = FILE_END;
         break;
     default:
+        errno = EINVAL;
         return -1;
     }
-    
-    return SetFilePointer((HANDLE)fd, (DWORD)offset, NULL, method);
+
+    if ((val=SetFilePointer((HANDLE)fd, (DWORD)offset, NULL, method)) == INVALID_SET_FILE_POINTER) {
+       errno = b_errno_win32;
+       return -1;
+    }
+    /* ***FIXME*** I doubt this works right */
+    return val;
 }
 
 int
 dup2(int, int)
 {
+    errno = ENOSYS;
     return -1;
 }
 
 
 #endif
+
+#endif //HAVE_MINGW
+
+#ifdef HAVE_MINGW
+/* syslog function, added by Nicolas Boichat */
+void closelog() {}
+#endif //HAVE_MINGW
